UI_STRINGS = {
    "error_no_magnets": "[yellow]No magnet links found.[/yellow]",
    "error_no_results": "[yellow]Could not find, try again.[/yellow]",
    "prompt_choose_indexer": "Choose an indexer:",
    "prompt_download_path": "Download path:",
    "prompt_search_query": "Search:",
    "prompt_select_result": "Select:",
    "status_fetching_magnets": "[bold green]Fetching magnet links for '{title}'...[/bold green]",
    "status_searching": "[bold green]Searching {indexer} for '{query}'...[/bold green]",
    "status_downloading_metadata": "[bold green]Downloading metadata...[/bold green]",
    "status_starting_download": "[bold green]Starting download: [cyan]{name}[/cyan][/bold green]",
    "info_size_files": "[yellow]Size:[/yellow] {size:.2f} MB | [yellow]Files:[/yellow] {count}",
    "status_download_complete": "\n[bold green]✓ Download complete: [cyan]{name}[/cyan][/bold green]",
    "info_saved_to": "[green]Saved to: [underline]{path}[/underline][/green]",
}



import asyncio
from typing import List
from urllib.parse import quote_plus

import httpx
from selectolax.parser import HTMLParser

from torrra.indexers.base import BaseIndexer
from torrra.types import Torrent


class Indexer(BaseIndexer):
    def search(self, query: str) -> List[Torrent]:
        normalized_query = quote_plus(query)
        url = self._get_url(f"/lmsearch?q={normalized_query}&cat=lmsearch")
        parser = self._get_parser(url)

        results = []
        urls = []

        nodes = parser.css("table tbody tr")
        for node in nodes:
            title_node = node.css_first("div.tt-name a")
            title = title_node.text(strip=True) if title_node else ""
            link = title_node.attributes.get("href") if title_node else ""
            size_node = node.css_first("td:nth-child(3)")
            size = size_node.text(strip=True) if size_node else ""

            if query not in title.lower() or link is None:
                continue

            results.append({"title": title, "size": size})
            urls.append(self._get_url(link))

        magnet_uris = asyncio.run(self._fetch_magnet_uris(urls))

        torrents: List[Torrent] = []
        for i, magnet_uri in enumerate(magnet_uris):
            if not magnet_uri:
                continue

            full_title = f"{results[i]['title']} {results[i]['size']}"
            torrents.append(Torrent(title=full_title, magnet_uri=magnet_uri))

        return torrents

    def _get_url(self, url: str) -> str:
        return f"https://magnetdl.hair{url}"

    async def _fetch_magnet_uris(self, urls: List[str]):
        async def fetch(client: httpx.AsyncClient, url: str):
            try:
                res = await client.get(url, timeout=10)
                parser = HTMLParser(res.text)
                node = parser.css_first("ul.download-links-dontblock li a")
                return node.attributes.get("href") if node else ""
            except:
                return None

        async with httpx.AsyncClient() as client:
            tasks = [fetch(client, url) for url in urls]
            return await asyncio.gather(*tasks)



import asyncio
from typing import List

import httpx
from selectolax.parser import HTMLParser

from torrra.indexers.base import BaseIndexer
from torrra.types import Torrent


class Indexer(BaseIndexer):
    def search(self, query: str) -> List[Torrent]:
        url = f"https://yts.mx/browse-movies/{query}/all/all/0/latest/0/all"
        parser = self._get_parser(url)

        titles_links: List[tuple[str, str]] = []

        nodes = parser.css(
            "div.browse-content div.browse-movie-wrap div.browse-movie-bottom"
        )
        for node in nodes:
            title_node = node.css_first("a.browse-movie-title")
            year_node = node.css_first("div.browse-movie-year")

            title = title_node.text(strip=True) if title_node else ""
            link = title_node.attributes.get("href") if title_node else ""
            year = year_node.text(strip=True) if year_node else ""

            if query not in title.lower() or link is None:
                continue

            titles_links.append((f"{title} {year}".strip(), link))

        magnets_list = asyncio.run(self._fetch_magnet_uris(titles_links))

        torrents = []
        for title, magnets in zip([t[0] for t in titles_links], magnets_list):
            for magnet in magnets:
                torrents.append(
                    Torrent(
                        title=f"{title} {magnet.title}", magnet_uri=magnet.magnet_uri
                    )
                )

        return torrents

    async def _fetch_magnet_uris(
        self, items: List[tuple[str, str]]
    ) -> List[List[Torrent]]:
        async def fetch(client: httpx.AsyncClient, url: str):
            try:
                res = await client.get(url, timeout=10)
                parser = HTMLParser(res.text)
                results = []

                nodes = parser.css("div.modal div.modal-torrent")
                for node in nodes:
                    resolution_node = node.css_first("div.modal-quality span")
                    resolution = (
                        resolution_node.text(strip=True) if resolution_node else ""
                    )

                    quality_info = resolution

                    quality_size_nodes = node.css("p.quality-size")
                    if len(quality_size_nodes) >= 2:
                        source = quality_size_nodes[0].text(strip=True)
                        size = quality_size_nodes[1].text(strip=True)
                        quality_info = f"{source} {resolution} {size}"

                    magnet_node = node.css_first("a.magnet")
                    magnet_uri = (
                        magnet_node.attributes.get("href") if magnet_node else ""
                    )

                    if not magnet_uri:
                        continue

                    results.append(Torrent(title=quality_info, magnet_uri=magnet_uri))
                return results

            except Exception:
                return []

        async with httpx.AsyncClient() as client:
            tasks = [fetch(client, url) for (_, url) in items]
            return await asyncio.gather(*tasks)



import questionary

from prompt_toolkit.shortcuts import CompleteStyle
from questionary import Choice
from typing import List
from rich.console import Console

from torrra.constants import UI_STRINGS
from torrra.downloader import download_magnet
from torrra.indexers import INDEXERS
from torrra.types import Torrent
from torrra.utils import get_indexer

console = Console()


def main() -> None:
    query = questionary.text(UI_STRINGS["prompt_search_query"]).ask()
    if not query:
        return

    indexer_name = questionary.select(
        UI_STRINGS["prompt_choose_indexer"], choices=list(INDEXERS.keys())
    ).ask()
    indexer = get_indexer(indexer_name)

    with console.status(
        UI_STRINGS["status_searching"].format(indexer=indexer_name, query=query)
    ):
        torrents: List[Torrent] = indexer.search(query)

    if not torrents:
        console.print(UI_STRINGS["error_no_results"])
        return

    torrent_choices = [
        Choice(title=torrent.title, value=torrent) for torrent in torrents
    ]
    selected_torrent: Torrent | None = questionary.select(
        UI_STRINGS["prompt_select_result"], choices=torrent_choices
    ).ask()
    if not selected_torrent:
        return

    save_path = questionary.path(
        UI_STRINGS["prompt_download_path"],
        only_directories=True,
        complete_style=CompleteStyle.COLUMN,
    ).ask()

    # initiate download
    download_magnet(selected_torrent.magnet_uri, save_path)



import re
import asyncio
from typing import List, Tuple
from urllib.parse import quote_plus

import httpx
from selectolax.parser import HTMLParser

from torrra.indexers.base import BaseIndexer
from torrra.types import Torrent


class Indexer(BaseIndexer):
    BASE_URL = ""
    def search(self, query: str) -> List[Torrent]:
        normalized_query = quote_plus(query)
        url = f"https://www.5movierulz.voto/search_movies?s={normalized_query}"
        parser = self._get_parser(url)

        results = []

        has_no_results = parser.css_first("div.content ul h1")
        if has_no_results:
            return results

        titles_links: List[Tuple[str, str]] = []

        nodes = parser.css("div.content ul li")
        for node in nodes:
            title_node = node.css_first("p b")
            link_node = node.css_first("a")
            title = title_node.text() if title_node else ""
            link = link_node.attributes.get("href") if link_node else ""

            if query.lower() not in title.lower() or not link:
                continue

            titles_links.append((title, link))

        magnets_list = asyncio.run(self._fetch_magnet_uris(titles_links))

        for title, magnets in zip([t[0] for t in titles_links], magnets_list):
            for magnet in magnets:
                results.append(
                    Torrent(
                        title=f"{title} {magnet.title}",
                        magnet_uri=magnet.magnet_uri
                    )
                )

        return results

    async def _fetch_magnet_uris(self, items: List[Tuple[str, str]]) -> List[List[Torrent]]:
        async def fetch(client: httpx.AsyncClient, url: str):
            res = await client.get(url, timeout=10)
            parser = HTMLParser(res.text)

            results = []

            a_nodes = parser.css("div.entry-content p a")
            nodes = [node for node in a_nodes if "GET THIS TORRENT" in node.text(strip=True)]

            for node in nodes:
                magnet_uri = node.attributes.get("href")
                if not magnet_uri:
                    continue

                title_node = node.css_first("small")
                title = title_node.text(strip=True) if title_node else ""
                formatted_title = re.sub(r'\b(\d+(\.\d+)?)\s*(gb|mb|kb)\b', lambda m: f"{m.group(1)} {m.group(3).upper()}", title)

                results.append(Torrent(title=formatted_title, magnet_uri=magnet_uri))

            return results

        async with httpx.AsyncClient() as client:
            tasks = [fetch(client, url) for (_, url) in items]
            return await asyncio.gather(*tasks)
